@@ -26,11 +26,25 @@ class ConformanceTestCase(unittest.TestCase):
 
     channel_layer = None
     expiry_delay = None
+    capacity_limit = None
+
+    @classmethod
+    def raiseSkip(cls, message):
+        """
+        Picks the right skip class based on what version of unittest we're using.
+        """ 
+        try:
+            import unittest2
+            SkipTest = unittest2.SkipTest
+        except ImportError:
+            SkipTest = unittest.SkipTest
+        raise SkipTest(message)
 
     @classmethod
     def setUpClass(cls):
+        # Don't let this actual class run, it's abstract
         if cls is ConformanceTestCase:
-            raise unittest.SkipTest("Skipping base class tests")
+            cls.raiseSkip("Skipping base class tests")
 
     def setUp(self):
         if self.channel_layer is None:
@@ -40,6 +54,14 @@ class ConformanceTestCase(unittest.TestCase):
         if "flush" in self.channel_layer.extensions:
             self.channel_layer.flush()
 
+    def skip_if_no_extension(self, extension):
+        """
+        Handy function for skipping things without an extension.
+        We can't use the decorators, as we need access to self.
+        """
+        if extension not in self.channel_layer.extensions:
+            self.raiseSkip("No %s extension" % extension)
+
     def test_send_recv(self):
         """
         Tests that channels can send and receive messages right.
@@ -60,16 +82,6 @@ class ConformanceTestCase(unittest.TestCase):
         self.assertEqual(channel, "sr_test2")
         self.assertEqual(message, {"value": "red"})
 
-    def test_unicode_channel_name(self):
-        """
-        Makes sure channel names can handle unicode
-        """
-        self.channel_layer.send("\u00a3_test", {"value": "blue"})
-        # Get just one first
-        channel, message = self.channel_layer.receive_many(["\u00a3_test"])
-        self.assertEqual(channel, "\u00a3_test")
-        self.assertEqual(message, {"value": "blue"})
-
     def test_message_expiry(self):
         """
         Tests that messages expire correctly.
@@ -138,9 +150,7 @@ class ConformanceTestCase(unittest.TestCase):
         """
         Tests that basic group addition and send works
         """
-        # Do skipping here as we need self
-        if "groups" not in self.channel_layer.extensions:
-            raise unittest.SkipTest("No groups extension")
+        self.skip_if_no_extension("groups")
         # Make a group and send to it
         self.channel_layer.group_add("tgroup", "tg_test")
         self.channel_layer.group_add("tgroup", "tg_test2")
@@ -163,9 +173,7 @@ class ConformanceTestCase(unittest.TestCase):
         """
         Tests that messages go away after a flush.
         """
-        # Do skipping here as we need self
-        if "flush" not in self.channel_layer.extensions:
-            raise unittest.SkipTest("No flush extension")
+        self.skip_if_no_extension("flush")
         # Send something to flush
         self.channel_layer.send("fl_test", {"value": "blue"})
         self.channel_layer.flush()
@@ -177,11 +185,8 @@ class ConformanceTestCase(unittest.TestCase):
         """
         Tests that groups go away after a flush.
         """
-        # Do skipping here as we need self
-        if "flush" not in self.channel_layer.extensions:
-            raise unittest.SkipTest("No flush extension")
-        if "groups" not in self.channel_layer.extensions:
-            raise unittest.SkipTest("No groups extension")
+        self.skip_if_no_extension("groups")
+        self.skip_if_no_extension("flush")
         # Add things to a group and send to it
         self.channel_layer.group_add("tfg_group", "tfg_test")
         self.channel_layer.send_group("tfg_group", {"value": "blue"})
@@ -195,9 +200,7 @@ class ConformanceTestCase(unittest.TestCase):
         Tests that group expiry is provided, and test it if it's less than
         20 seconds.
         """
-        # Do skipping here as we need self
-        if "groups" not in self.channel_layer.extensions:
-            raise unittest.SkipTest("No groups extension")
+        self.skip_if_no_extension("groups")
         # Check group expiry is provided, and see if we can continue
         expiry = getattr(self.channel_layer, "group_expiry", None)
         if expiry is None:
@@ -213,3 +216,22 @@ class ConformanceTestCase(unittest.TestCase):
         channel, message = self.channel_layer.receive_many(["tge_test"])
         self.assertIs(channel, None)
         self.assertIs(message, None)
+
+    def test_capacity(self):
+        """
+        Tests that the capacity limiter on send() raises ChannelFull
+        after the right number of messages. Only runs if capacity_limit is set.
+        """
+        if self.capacity_limit is None:
+            self.raiseSkip("No test capacity specified")
+        for _ in range(self.capacity_limit):
+            self.channel_layer.send("cap_test", {"hey": "there"})
+        with self.assertRaises(self.channel_layer.ChannelFull):
+            self.channel_layer.send("cap_test", {"hey": "there"})
+
+    def test_exceptions(self):
+        """
+        Tests that the two exception classes exist on the channel layer
+        """
+        self.assertTrue(hasattr(self.channel_layer, "MessageTooLarge"))
+        self.assertTrue(hasattr(self.channel_layer, "ChannelFull"))
