@@ -13,11 +13,13 @@ from collections import deque
 class ChannelLayer(object):
     """
     In memory channel layer object; a single one is instantiated as
-    "channel_layer" for easy shared use.
+    "channel_layer" for easy shared use. Only allows global capacity config.
     """
 
-    def __init__(self, expiry=60):
+    def __init__(self, expiry=60, group_expiry=86400, capacity=100):
         self.expiry = expiry
+        self.capacity = capacity
+        self.group_expiry = group_expiry
         self.thread_lock = threading.Lock()
         # Storage for state
         self._channels = {}
@@ -27,6 +29,9 @@ class ChannelLayer(object):
 
     extensions = ["groups", "flush"]
 
+    class ChannelFull(Exception):
+        pass
+
     class MessageTooLarge(Exception):
         pass
 
@@ -37,7 +42,10 @@ class ChannelLayer(object):
         assert isinstance(channel, six.text_type), "%s is not text" % channel
         # Add it to a deque for the appropriate channel name
         with self.thread_lock:
-            self._channels.setdefault(channel, deque()).append((
+            queue = self._channels.setdefault(channel, deque())
+            if len(queue) >= self.capacity:
+                raise self.ChannelFull(channel)
+            queue.append((
                 time.time() + self.expiry,
                 message,
             ))
@@ -74,26 +82,35 @@ class ChannelLayer(object):
         # Both should be text
         assert isinstance(group, six.text_type), "%s is not text" % group
         assert isinstance(channel, six.text_type), "%s is not text" % channel
-        # Add to group set
-        self._groups[group] = self._groups.get(group, set()).union({channel})
+        # Add to group dict
+        with self.thread_lock:
+            self._groups.setdefault(group, {})
+            self._groups[group][channel] = time.time()
 
     def group_discard(self, group, channel):
         # Both should be text
         assert isinstance(group, six.text_type), "%s is not text" % group
         assert isinstance(channel, six.text_type), "%s is not text" % channel
         # Remove from group set
-        if group in self._groups:
-            self._groups[group].discard(channel)
-            if not self._groups[group]:
-                del self._groups[group]
+        with self.thread_lock:
+            if group in self._groups:
+                if channel in self._groups[group]:
+                    del self._groups[group][channel]
+                if not self._groups[group]:
+                    del self._groups[group]
 
     def send_group(self, group, message):
         # Check types
         assert isinstance(message, dict), "message is not a dict"
         assert isinstance(group, six.text_type), "%s is not text" % group
+        # Run clean
+        self._clean_expired()
         # Send to each channel
         for channel in self._groups.get(group, set()):
-            self.send(channel, message)
+            try:
+                self.send(channel, message)
+            except self.ChannelFull:
+                pass
 
     ### ASGI Flush API ###
 
@@ -105,10 +122,11 @@ class ChannelLayer(object):
 
     def _clean_expired(self):
         """
-        Goes through all messages and removes those that are expired.
+        Goes through all messages and groups and removes those that are expired.
         Any channel with an expired message is removed from all groups.
         """
         with self.thread_lock:
+            # Channel cleanup
             for channel, queue in list(self._channels.items()):
                 remove = False
                 # See if it's expired
@@ -121,13 +139,21 @@ class ChannelLayer(object):
                 # Is the channel now empty and needs deleting?
                 if not queue:
                     del self._channels[channel]
+            # Group cleanup
+            for group, channels in list(self._groups.items()):
+                for channel, added in list(channels.items()):
+                    if added < (time.time() - self.group_expiry):
+                        del self._groups[group][channel]
+                        if not self._groups[group]:
+                            del self._groups[group]
 
     def _remove_from_groups(self, channel):
         """
         Removes a channel from all groups. Used when a message on it expires.
         """
-        for  channels in self._groups.values():
-            channels.discard(channel)
+        for channels in self._groups.values():
+            if channel in channels:
+                del channels[channel]
 
 # Global single instance for easy use
 channel_layer = ChannelLayer()
